
// Dieser Modus ist ein Hauptschaltermodus mit Secondary Protection 
// Secondary Protection löst aus, wenn OVP und LVP wegfällt.
// OVP und LVP fällt wegbei:
// - Sehr tiefe Entladung
// - Sehr hohe Spannung
// - Übertemperatur
// - je nach Liproeinstellung möglicherweise auch wenn sowohl Untertemperaturschutz für Ladung und für Last erreicht ist
// - je nach Liproeinstellung möglicherweise auch wenn sowohl Überttemperaturschutz für Ladung und für Last erreicht ist
// - Die letzten beiden Positionen können vielleicht ungewollt sein.

// OVP UND LVP Signal gleichzeitig:
// Es wurde eine Verzögerung von ca. 30 Sekunden implementiert. So kann noch problemlos ein Testjumper auf die Lipro gesteckt werden und die 
// einzelnennen Funktionen zu prüfen. Außerdem ist es eventuell für die Prametrierung hilfreich, wenn nicht sofort ausgeht
// Auch wäre es hilfreich um zum Beispiel die Ursache über Modbus abfragen heruas zu bekommen

// 
// Fault Input:
// Hier ohne Verzögerung um schnell auf kurzschluss reagieren zu können
// Ansonsten wie Modus 0



#include "stdio.h"
#include "mode_secondaryprotection.h"
#include "button.h"
#include "relais.h"
#include "main.h"
#include "leds.h"
#include "buzzer.h"
#include "voltage_meas.h"


typedef enum SECONDARYPROTECTION_State_enum 
{
  SECONDARYPROTECTION_OFF, 
  SECONDARYPROTECTION_ON,
  SECONDARYPROTECTION_MANUAL_ON, 
  SECONDARYPROTECTION_ERROR
} SECONDARYPROTECTION_state_t;


static SECONDARYPROTECTION_state_t smState;


static void SECONDARYPROTECTION_SM_Off(void)
{
  int faultInput;
  int lvpAndOvpInput;

  if (HAL_GPIO_ReadPin(GPIO_INPUT_FAULT_GPIO_Port, GPIO_INPUT_FAULT_Pin) == GPIO_PIN_RESET)       
  {
    faultInput = 1;
  }
  else
  {
    faultInput = 0;
  }

  if ((HAL_GPIO_ReadPin(GPIO_INPUT_LVP_GPIO_Port, GPIO_INPUT_LVP_Pin) == GPIO_PIN_SET) &&  (HAL_GPIO_ReadPin(GPIO_INPUT_OVP_GPIO_Port, GPIO_INPUT_OVP_Pin) == GPIO_PIN_SET))
  {
    lvpAndOvpInput = 1;
  }else {
    lvpAndOvpInput = 0;
  }

  


  //Prüfe auf Wechsel des Modus AUTO / SM ON
  if (BUTTON_GetMode() == BUTTON_AUTO)
  {
    if (faultInput == 0) 
    {
      RELAIS_SetPuls();
      BUZZER_Beep(BUZZER_ON_TIME_CONFIRM);
      LEDS_GN_Blink_Start(LED_GN_ON_TIME_ON_MODE, LED_GN_OFF_TIME);
      printf("NEW_STATE: SECONDARYPROTECTION_ON\n");
      smState = SECONDARYPROTECTION_ON;

    }
    else   
    {
      //Wechsel nicht möglich. Fehler Eingang aktiv
      BUZZER_Beep(BUZZER_ON_TIME_REJECT);
      BUTTON_SetModeOff();
      //LEDS_RT_Blink_Start(LED_RT_ON_TIME_WARN_FAULT_INPUT, LED_GN_OFF_TIME); //Fehler Anzeigen
	  LEDS_RT_BlinkCode_Start(BLINK_CODE_ERROR_FAULT_INPUT, LED_RT_ON_TIME_WARN_FAULT_INPUT, LED_RT_OFF_TIME, LED_RT_OFF_TIME *5); //Fehler Anzeigen
      printf("NEW_STATE: SECONDARYPROTECTION_ERROR\n");
      smState =SECONDARYPROTECTION_ERROR;
    }
  }


  //Prüfe auf Wechsel in MANUAL ON Mode
  //Keine Fehlerüberprüfungen. In diesem Modus werdem alle Alarme ignoriert.
  if (BUTTON_GetMode() == BUTTON_MANUAL_ON)
  {
      
      RELAIS_SetPuls();
      BUZZER_Alarm_Start(BUZZER_ON_TIME_ALARM_MANUAL_MODE, BUZZER_OFF_TIME);
      LEDS_GN_On();
      LEDS_RT_BlinkCode_Start(BLINK_CODE_WARN_MANUAL, LED_RT_ON_TIME_WARN_MANUAL_MODE, LED_RT_OFF_TIME, LED_RT_OFF_TIME * 5); //Fehler Anzeigen
      printf("NEW_STATE: SECONDARYPROTECTION_MANUAL_ON\n");
      smState = SECONDARYPROTECTION_MANUAL_ON;
  }



}

static void SECONDARYPROTECTION_SM_On(void)
{
  int faultInput = 0;
  int lvpAndOvpInput = 0;
  static int lvpAndOvpInputTimeCounter = 0;
  static int oldtime;

  if (HAL_GPIO_ReadPin(GPIO_INPUT_FAULT_GPIO_Port, GPIO_INPUT_FAULT_Pin) == GPIO_PIN_RESET)       
  {
    faultInput = 1;
  }
  else
  {
    faultInput = 0;
  }
  
  if ((HAL_GPIO_ReadPin(GPIO_INPUT_LVP_GPIO_Port, GPIO_INPUT_LVP_Pin) == GPIO_PIN_SET) &&  (HAL_GPIO_ReadPin(GPIO_INPUT_OVP_GPIO_Port, GPIO_INPUT_OVP_Pin) == GPIO_PIN_SET))
  {
    if (HAL_GetTick() != oldtime)
    {
      lvpAndOvpInputTimeCounter++;
      if (lvpAndOvpInputTimeCounter > 30000)
      {
        lvpAndOvpInput = 1;
        lvpAndOvpInputTimeCounter=0;
      }
      oldtime = HAL_GetTick();
    }
  }
  else 
  {
    lvpAndOvpInputTimeCounter = 0;
    lvpAndOvpInput = 0;
  }

  

  // Prüfe Wechsel in off mode
  if (BUTTON_GetMode() == BUTTON_OFF)
  {
    //Ausschalten muss immer möglich sein
    RELAIS_ResetPuls();
    BUZZER_Beep(BUZZER_ON_TIME_CONFIRM); //Bestätigung
    LEDS_GN_Off();
    LEDS_RT_Off();
    printf("NEW_STATE: SECONDARYPROTECTION_OFF\n");
    smState = SECONDARYPROTECTION_OFF;
  }

  //Prüfe auf Fehlermode
  if (VOLTAGE_MEAS_GetLimitAlarm() == 1)
  {
	  RELAIS_ResetPuls();
	  BUZZER_Beep(BUZZER_ON_TIME_REJECT); //Warnung
	  LEDS_GN_Off();
	  LEDS_RT_BlinkCode_Start(BLINK_CODE_ERROR_VOLTAGE_DROP, LED_RT_ON_TIME_WARN_VOLTAGE_DROP, LED_RT_OFF_TIME, LED_RT_OFF_TIME *5); //Fehler Anzeigen
	  BUTTON_SetModeOff(); //Damit nicht von alleine wieder eingeschaltet wird
	  printf("FAULT VOLTAGE DROP DETECTED!\n");
	  printf("NEW_STATE: SECONDARYPROTECTION_ERROR\n");
	  smState = SECONDARYPROTECTION_ERROR;  
	  VOLTAGE_MEAS_ResetCounter(); // Damit Fehlerzähler bei Neustart auf 0
  }


  if (faultInput == 1)
  {
    RELAIS_ResetPuls();
    BUZZER_Beep(BUZZER_ON_TIME_REJECT); //Warnung
    LEDS_GN_Off();
    LEDS_RT_BlinkCode_Start(BLINK_CODE_ERROR_FAULT_INPUT, LED_RT_ON_TIME_WARN_FAULT_INPUT, LED_RT_OFF_TIME, LED_RT_OFF_TIME *5); //Fehler Anzeigen
    BUTTON_SetModeOff(); //Damit nicht von alleine wieder eingeschaltet wird
    printf("FAULT INPUT EVENT DETECTED!\n");
    printf("NEW_STATE: SECONDARYPROTECTION_ERROR\n");
    smState = SECONDARYPROTECTION_ERROR;
  }

  if (lvpAndOvpInput == 1)
  {
    RELAIS_ResetPuls();
    BUZZER_Beep(BUZZER_ON_TIME_REJECT); //Warnung
    LEDS_GN_Off();
    LEDS_RT_BlinkCode_Start(BLINK_CODE_ERROR_OVP_LVP, LED_RT_ON_TIME_WARN_OVP_AND_LVP_INPUT, LED_RT_OFF_TIME, LED_RT_OFF_TIME *5); //Fehler Anzeigen
    BUTTON_SetModeOff(); //Damit nicht von alleine wieder eingeschaltet wird
    printf("BMS SECONDARY PROTECTION FAULT EVENT DETECTED (LVP & OVP )!\n");
    printf("NEW_STATE: SECONDARYPROTECTION_ERROR\n");
    smState = SECONDARYPROTECTION_ERROR;
  }


}

static void SECONDARYPROTECTION_SM_ManualOn(void)
{
  // Prüfe Wechsel in off mode
  if (BUTTON_GetMode() == BUTTON_OFF)
  {
    //Ausschalten muss immer möglich sein
    RELAIS_ResetPuls();
    BUZZER_Alarm_Stop();
    LEDS_GN_Off();
    LEDS_RT_Off();
    printf("NEW_STATE: SECONDARYPROTECTION_OFF\n");
    smState = SECONDARYPROTECTION_OFF;
  }

}

static void SECONDARYPROTECTION_SM_Error(void)
{
  int faultInput;
  int lvpAndOvpInput;

  if (HAL_GPIO_ReadPin(GPIO_INPUT_FAULT_GPIO_Port, GPIO_INPUT_FAULT_Pin) == GPIO_PIN_RESET)       
  {
    faultInput = 1;
  }
  else
  {
    faultInput = 0;
  }



  //Prüfe auf Wechsel des Modus AUTO / SM ON
  if (BUTTON_GetMode() == BUTTON_AUTO)
  {
    if (faultInput == 0) 
    {
      RELAIS_SetPuls();
      BUZZER_Beep(BUZZER_ON_TIME_CONFIRM);
      LEDS_GN_Blink_Start(LED_GN_ON_TIME_ON_MODE, LED_GN_OFF_TIME);
      LEDS_RT_Off(); //Fehler löschen
      printf("NEW_STATE: SECONDARYPROTECTION_ON\n");
      smState = SECONDARYPROTECTION_ON;
    }
    else
    {
      //Wechsel nicht möglich. Fehler Eingang weiterhin aktiv
      BUZZER_Beep(BUZZER_ON_TIME_REJECT);
      BUTTON_SetModeOff();
    }
  }

  //Prüfe auf Wechsel in MANUAL ON Mode
  //Keine Fehlerüberprüfungen. In diesem Modus werdem alle Alarme ignoriert.
  if (BUTTON_GetMode() == BUTTON_MANUAL_ON)
  {
      
      RELAIS_SetPuls();
      BUZZER_Alarm_Start(BUZZER_ON_TIME_ALARM_MANUAL_MODE, BUZZER_OFF_TIME);
      LEDS_GN_On();
      LEDS_RT_Off();
      LEDS_RT_BlinkCode_Start(BLINK_CODE_WARN_MANUAL, LED_RT_ON_TIME_WARN_MANUAL_MODE, LED_RT_OFF_TIME, LED_RT_OFF_TIME *5); //Fehler Anzeigen
      printf("NEW_STATE: SECONDARYPROTECTION_MANUAL_ON\n");
      smState = SECONDARYPROTECTION_MANUAL_ON;
  }



}



void MODE_SECONDARYPROTECTION_Exec(void)
{

  

  switch (smState)
  {
    case SECONDARYPROTECTION_OFF:
      SECONDARYPROTECTION_SM_Off();
    break;

    case SECONDARYPROTECTION_ON:
      SECONDARYPROTECTION_SM_On();
    break;

    case SECONDARYPROTECTION_MANUAL_ON:
      SECONDARYPROTECTION_SM_ManualOn();
    break;

    case SECONDARYPROTECTION_ERROR:
      SECONDARYPROTECTION_SM_Error();
    break;

    default:
    break;
  }
}


